@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  gp-gc.c: An opaque object, holding current drawing state
  *
@@ -19,8 +19,7 @@
  *  Authors:
  *    Lauris Kaplinski <lauris@ximian.com>
  *
- *  Copyright (C) 2000-2002 Ximian, Inc. and authors
- *
+ *  Copyright (C) 2000-2003 Ximian, Inc. and authors
  */
  
 /*
@@ -31,8 +30,6 @@
  *
  */
 
-#define _GP_FONTMAP_C_
-
 #include <config.h>
 #include <math.h>
 #include <string.h>
@@ -50,9 +47,9 @@
 #include <libart_lgpl/art_vpath_svp.h>
 #include <libart_lgpl/art_svp_vpath_stroke.h>
 
-#include "gp-path.h"
-#include "gp-gc.h"
-#include "gp-gc-private.h"
+#include <libgnomeprint/gp-path.h>
+#include <libgnomeprint/gp-gc.h>
+#include <libgnomeprint/gp-gc-private.h>
 
 #define GP_GC_EPSILON 1e-18
 #define GP_GC_EQ(a,b) (fabs (a - b) < GP_GC_EPSILON)
@@ -89,7 +86,7 @@ struct _GPCtx {
 	ArtSVP * clipsvp;
 	gboolean privateclip;
 
-	gpointer data;
+	GObject *user_data;
 };
 
 static GPCtx * gp_ctx_new (void);
@@ -374,9 +371,10 @@ gp_gc_set_linewidth (GPGC * gc, gdouble width)
 
 	ctx = (GPCtx *) gc->ctx->data;
 
+	width = (fabs (width * ctx->ctm[0]) + fabs (width * ctx->ctm[1]) +
+		 fabs (width * ctx->ctm[2]) + fabs (width * ctx->ctm[3])) / 2;
 	if (!GP_GC_EQ (width, ctx->linewidth)) {
-		ctx->linewidth = (fabs (width * ctx->ctm[0]) + fabs (width * ctx->ctm[1]) +
-				  fabs (width * ctx->ctm[2]) + fabs (width * ctx->ctm[3])) / 2;
+		ctx->linewidth = width;
 		ctx->line_flag = GP_GC_FLAG_CHANGED;
 	}
 
@@ -463,7 +461,7 @@ gp_gc_get_linecap (GPGC * gc)
 {
 	g_return_val_if_fail (gc != NULL, -1);
 
-	return ((GPCtx *) gc->ctx->data)->linejoin;
+	return ((GPCtx *) gc->ctx->data)->linecap;
 }
 
 gint
@@ -494,10 +492,12 @@ gp_gc_set_dash (GPGC * gc, int num_values, const gdouble * values, gdouble offse
 
 	ctx = (GPCtx *) gc->ctx->data;
 
-	if ((ctx->dash.dash) && (ctx->privatedash)) g_free (ctx->dash.dash);
+	if ((ctx->dash.dash) && (ctx->privatedash))
+		g_free (ctx->dash.dash);
 	ctx->dash.n_dash = num_values;
 	ctx->dash.offset = offset;
 	if (values != NULL) {
+		/* FIXME: Shouldn't this set ctx->privatedash?  */
 		ctx->dash.dash = g_new (gdouble, num_values);
 		memcpy (ctx->dash.dash, values, num_values * sizeof (gdouble));
 	} else {
@@ -868,22 +868,35 @@ gp_gc_get_clipsvp (GPGC * gc)
 	return ((GPCtx *) gc->ctx->data)->clipsvp;
 }
 
-gint
-gp_gc_set_data (GPGC * gc, gpointer data)
+static void
+cb_user_data_lost (GPCtx *ctx, GObject *where_the_object_was)
 {
-	g_return_val_if_fail (gc != NULL, 1);
-
-	((GPCtx *) gc->ctx->data)->data = data;
-
-	return 0;
+	ctx->user_data = NULL;
+	g_warning ("lost it");
 }
 
-gpointer
-gp_gc_get_data (GPGC * gc)
+void
+gp_gc_set_user_data (GPGC *gc, GObject *user_data)
+{
+	GPCtx *ctx;
+
+	g_return_if_fail (gc != NULL);
+
+	ctx = (GPCtx *) gc->ctx->data;
+	if (NULL != ctx->user_data)
+		g_object_weak_unref (ctx->user_data,
+			(GWeakNotify) cb_user_data_lost, ctx);
+	if (NULL != (ctx->user_data = user_data))
+		g_object_weak_ref (ctx->user_data,
+			(GWeakNotify) cb_user_data_lost, ctx);
+}
+
+GObject *
+gp_gc_get_user_data (GPGC const *gc)
 {
 	g_return_val_if_fail (gc != NULL, NULL);
 
-	return ((GPCtx *) gc->ctx->data)->data;
+	return ((GPCtx *) gc->ctx->data)->user_data;
 }
 
 /* Private stuff */
@@ -914,7 +927,7 @@ gp_ctx_new (void)
 	ctx->dash_flag = GP_GC_FLAG_UNSET;
 	ctx->privatedash = FALSE;
 
-	ctx->font = gnome_font_find_closest ("Helvetica", 12.0);
+	ctx->font = gnome_font_find_closest ("Sans Regular", 12.0);
 	ctx->font_flag = GP_GC_FLAG_UNSET;
 
 	g_return_val_if_fail (ctx->font != NULL, NULL);
@@ -924,7 +937,7 @@ gp_ctx_new (void)
 	ctx->clipsvp = NULL;
 	ctx->privateclip = FALSE;
 
-	ctx->data = NULL;
+	ctx->user_data = NULL;
 
 	return ctx;
 }
@@ -969,7 +982,9 @@ gp_ctx_duplicate (const GPCtx * src)
 	ctx->clipsvp = src->clipsvp;
 	ctx->privateclip = FALSE;
 
-	ctx->data = src->data;
+	if (NULL != (ctx->user_data = src->user_data))
+		g_object_weak_ref (ctx->user_data,
+			(GWeakNotify) cb_user_data_lost, ctx);
 
 	return ctx;
 }
@@ -987,6 +1002,9 @@ gp_ctx_destroy (GPCtx * ctx)
 		art_svp_free (ctx->clipsvp);
 	}
 
+	if (NULL != ctx->user_data)
+		g_object_weak_unref (ctx->user_data,
+			(GWeakNotify) cb_user_data_lost, ctx);
 	g_object_unref (G_OBJECT (ctx->font));
 
 	gp_path_unref (ctx->currentpath);
@@ -1032,7 +1050,8 @@ gp_ctx_clip (GPCtx * ctx, ArtWindRule wind)
 		svp2 = art_svp_intersect (ctx->clipsvp, svp1);
 		g_assert (svp2 != NULL);
 		art_svp_free (svp1);
-		if (ctx->privateclip) art_svp_free (ctx->clipsvp);
+		if (ctx->privateclip)
+			art_svp_free (ctx->clipsvp);
 		ctx->clipsvp = svp2;
 		ctx->privateclip = TRUE;
 	} else {
@@ -1072,7 +1091,8 @@ gp_gc_matrix_equal (const gdouble * a, const gdouble * b)
 	gint i;
 
 	for (i = 0; i < 6; i++) {
-		if (fabs (a[i] - b[i]) > GP_GC_EPSILON) return FALSE;
+		if (fabs (a[i] - b[i]) > GP_GC_EPSILON)
+			return FALSE;
 	}
 
 	return TRUE;
