@@ -269,7 +269,8 @@ load_paper_sizes (PRINTER_INFO_5 *info, DEVMODE *pDevmode, GPANode *parent)
 	GPANode *node, *size;
 	DWORD paper_cnt, i;
 	LPTSTR paper_names;
-	gchar paper_name[WIN32_MAX_PAPER_NAME_LEN + 1], *paper_nick;
+	gchar *paper_name, *paper_nick;
+	gchar paper_buf[WIN32_MAX_PAPER_NAME_LEN + 1];
 	gchar fp_buf[G_ASCII_DTOSTR_BUF_SIZE];
 	POINT *paper_sizes;
 	WORD *papers;
@@ -282,31 +283,37 @@ load_paper_sizes (PRINTER_INFO_5 *info, DEVMODE *pDevmode, GPANode *parent)
 
 	paper_cnt = DeviceCapabilities (info->pPrinterName, info->pPortName,
 					DC_PAPERNAMES, NULL, NULL);
-	if (!paper_cnt)
+	if (paper_cnt <= 0)
 		return;
 
 	paper_names = g_malloc (paper_cnt * sizeof(gchar) * WIN32_MAX_PAPER_NAME_LEN);
 	paper_sizes = g_malloc (paper_cnt * sizeof(POINT));
 	papers = g_malloc (paper_cnt * sizeof(WORD));
-	DeviceCapabilities (info->pPrinterName, info->pPortName,
-			    DC_PAPERNAMES, paper_names, NULL);
-	DeviceCapabilities (info->pPrinterName, info->pPortName,
-			    DC_PAPERS, (LPTSTR) papers, NULL);
-	DeviceCapabilities (info->pPrinterName, info->pPortName,
-			    DC_PAPERSIZE, (LPTSTR) paper_sizes, NULL);
+	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
+				DC_PAPERNAMES, paper_names, NULL) != paper_cnt)
+		g_warning ("Expected %d DC_PAPERNAMES", paper_cnt);
+	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
+				DC_PAPERS, (LPTSTR) papers, NULL) != paper_cnt)
+		g_warning ("Expected %d DC_PAPERS", paper_cnt);
+	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
+				DC_PAPERSIZE, (LPTSTR) paper_sizes, NULL)
+	    != paper_cnt)
+		g_warning ("Expected %d DC_PAPERS", paper_cnt);
 
-	paper_name[WIN32_MAX_PAPER_NAME_LEN] = '\0';
+	paper_buf[WIN32_MAX_PAPER_NAME_LEN] = '\0';
 	for (i = 0; i < paper_cnt; ++i) {
 		gchar *height;
 		gchar *width;
 
-		strncpy (paper_name, paper_names + i * WIN32_MAX_PAPER_NAME_LEN,
+		strncpy (paper_buf, paper_names + i * WIN32_MAX_PAPER_NAME_LEN,
 			 WIN32_MAX_PAPER_NAME_LEN);
+		paper_name = g_locale_to_utf8 (paper_buf, -1, NULL, NULL, NULL);
 		paper_nick = get_paper_nick (papers[i]);
 		size = gpa_option_item_new (node,
 					    paper_nick,
 					    paper_name);
 		g_free (paper_nick);
+		g_free (paper_name);
 
 		width  = g_strdup_printf ("%smm",
 					  g_ascii_dtostr (fp_buf,
@@ -340,7 +347,8 @@ load_paper_sources (PRINTER_INFO_5 *info, DEVMODE *pDevmode, GPANode *parent)
 	GPANode *node;
 	DWORD bin_cnt, i;
 	LPTSTR bin_names;
-	gchar bin_name[WIN32_MAX_PAPER_NAME_LEN + 1], *bin_nick;
+	gchar *bin_name, *bin_nick;
+	gchar  bin_buf[WIN32_MAX_PAPER_NAME_LEN + 1];
 	WORD *bins;
 
 	bin_nick = get_bin_nick(pDevmode->dmDefaultSource);
@@ -351,27 +359,31 @@ load_paper_sources (PRINTER_INFO_5 *info, DEVMODE *pDevmode, GPANode *parent)
 
 	bin_cnt = DeviceCapabilities (info->pPrinterName, info->pPortName,
 				      DC_BINNAMES, NULL, NULL);
-	if (!bin_cnt)
+	if (bin_cnt <= 0)
 		return;
 
 	bin_names = g_malloc (bin_cnt * sizeof(gchar) * WIN32_MAX_BIN_NAME_LEN);
 	bins = g_malloc (bin_cnt * sizeof(WORD));
-	DeviceCapabilities (info->pPrinterName, info->pPortName,
-			    DC_BINNAMES, bin_names, NULL);
-	DeviceCapabilities (info->pPrinterName, info->pPortName,
-			    DC_BINS, (LPTSTR) bins, NULL);
+	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
+				DC_BINNAMES, bin_names, NULL) != bin_cnt)
+		g_warning ("Expected %d DC_BINNAMES", bin_cnt);
+	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
+				DC_BINS, (LPTSTR) bins, NULL) != bin_cnt)
+		g_warning ("Expected %d DC_BINS", bin_cnt);
 
-	bin_name[WIN32_MAX_BIN_NAME_LEN] = '\0';
+	bin_buf[WIN32_MAX_BIN_NAME_LEN] = '\0';
 	for (i = 0; i < bin_cnt; ++i) {
 		GPANode *bin;
 
-		strncpy (bin_name, bin_names + i * WIN32_MAX_BIN_NAME_LEN,
+		strncpy (bin_buf, bin_names + i * WIN32_MAX_BIN_NAME_LEN,
 			 WIN32_MAX_BIN_NAME_LEN);
+		bin_name = g_locale_to_utf8 (bin_buf, -1, NULL, NULL, NULL);
 		bin_nick = get_bin_nick (bins[i]);
 		bin = gpa_option_item_new (node,
 					   bin_nick,
 					   bin_name);
 		g_free (bin_nick);
+		g_free (bin_name);
 	}
 
 	gpa_node_reverse_children (node);
@@ -388,7 +400,7 @@ load_other_settings (PRINTER_INFO_5 *info, DEVMODE *pDevmode, GPANode *model)
 	job = gpa_node_lookup (model, "Options.Output.Job");
 
 	if (DeviceCapabilities (info->pPrinterName, info->pPortName,
-				DC_DUPLEX, NULL, NULL)) {
+				DC_DUPLEX, NULL, NULL) > 0) {
 		gpa_option_string_new (job, "Duplex",
 				       pDevmode->dmDuplex == DMDUP_SIMPLEX ? "false" : "true");
 		gpa_option_string_new (job, "Tumble",
@@ -420,12 +432,15 @@ get_model (PRINTER_INFO_5 *info, DEVMODE *pDevmode)
 	GPANode *output;
 	gchar *xml;
 	gchar *id;
+	gchar *pname;
 
-	id = g_strdup_printf ("Win32-%s", info->pPrinterName);
-	xml = g_strdup_printf ((char *)model_xml_template, id, info->pPrinterName);
+	pname = g_locale_to_utf8 (info->pPrinterName, -1, NULL, NULL, NULL);
+	id = g_strdup_printf ("Win32-%s", pname);
+	xml = g_strdup_printf ((char *)model_xml_template, id, pname);
 	model = gpa_model_new_from_xml_str (xml);
 	g_free (xml);
 	g_free (id);
+	g_free (pname);
 
 	output = gpa_node_lookup (model, "Options.Output");
 	media  = gpa_node_lookup (model, "Options.Output.Media");
@@ -450,6 +465,7 @@ append_printer (GPAList *printers_list, PRINTER_INFO_5 *info, const gchar *def_p
 	GPANode *printer = NULL;
 	GPAModel *model = NULL;
 	gboolean retval = FALSE;
+	gchar *pname;
 
 	if (!OpenPrinter (info->pPrinterName, &hPrinter, NULL))
 		return FALSE;
@@ -480,7 +496,8 @@ append_printer (GPAList *printers_list, PRINTER_INFO_5 *info, const gchar *def_p
 		goto append_printer_exit;
 	}
 
-	printer = gpa_printer_new_stub (info->pPrinterName, info->pPrinterName, module_path);
+	pname = g_locale_to_utf8 (info->pPrinterName, -1, NULL, NULL, NULL);
+	printer = gpa_printer_new_stub (info->pPrinterName, pname, module_path);
 	if (printer == NULL) {
 		g_warning ("gpa_printer_new failed\n");
 		goto append_printer_exit;
@@ -501,12 +518,13 @@ append_printer (GPAList *printers_list, PRINTER_INFO_5 *info, const gchar *def_p
 
 append_printer_exit:
 	if (retval == FALSE) {
-		g_warning ("The printer %s could not be created\n", info->pPrinterName);
+		g_warning ("The printer %s could not be created\n", pname);
 
 		my_gpa_node_unref (printer);
 		my_gpa_node_unref (GPA_NODE (model));
 		my_gpa_node_unref (settings);
 	}
+	g_free (pname);
 
 	return retval;
 }
@@ -612,11 +630,15 @@ cb_update_status (gpointer data)
 	HANDLE hPrinter;
 	PRINTER_INFO_2 *info2_arr = NULL;
 	DWORD buf_size = 0;
+	gboolean res;
 
 	if (!printer->polling)
 		return FALSE;
 
-	if (!OpenPrinter (printer->name, &hPrinter, NULL))
+	str = g_locale_from_utf8(printer->name, -1, NULL, NULL, NULL);
+	res = OpenPrinter (str, &hPrinter, NULL);
+	g_free (str);
+	if (!res)
 		return FALSE;
 
 	while (!GetPrinter (hPrinter, 2, (LPBYTE) info2_arr,
@@ -649,8 +671,10 @@ cb_update_status (gpointer data)
 		node = GPA_NODE (gpa_state_new ("Location"));
 		gpa_node_attach (state, node);
 	}
-	gpa_node_set_value (node, info2_arr->pLocation);
-
+	str = g_locale_to_utf8 (info2_arr->pLocation, -1, NULL, NULL, NULL);
+	gpa_node_set_value (node, str);
+	
+	g_free (str);
 	g_free (info2_arr);
 
 	return TRUE;
