@@ -25,7 +25,6 @@
 
 #include "config.h"
 
-#include <popt.h>
 #include <glib.h>
 #include <string.h>
 #include <unistd.h>
@@ -43,27 +42,26 @@
 
 GMainLoop * loop;
 gint callback_count = 0;
-gint num = 0;
-gchar *num_str = NULL;
+gint num = G_MININT;
 gboolean debug = FALSE;
 gboolean dump_config = FALSE;
 gboolean dump_root = FALSE;
+gchar **args = NULL;
 GList * cmd_line_nodes = NULL;
 
 #define max_num ((sizeof (num_table) / sizeof (num_table[0]))-1)
 
-static struct poptOption options[] = {
-	{ "num", '\0', POPT_ARG_STRING, &num_str,   0,
-	  "Num number of functions to run, where nn is the num number", "#"},
-	{ "dump-config",     '\0', POPT_ARG_NONE, &dump_config,   0,
+static const GOptionEntry options[] = {
+	{ "num", '\0', 0, G_OPTION_ARG_INT, &num,
+	  "Num number of functions to run, where nn is the num number", "nn"},
+	{ "dump-config",     '\0', 0, G_OPTION_ARG_NONE, &dump_config,
 	  "Dump the GnomePrintConfig object to the console",  NULL},
-	{ "dump-root",     '\0', POPT_ARG_NONE,   &dump_root,   0,
+	{ "dump-root",     '\0', 0, G_OPTION_ARG_NONE,   &dump_root,
 	  "Dump the GpaRoot node to the console",  NULL},
-	{ "debug",    '\0', POPT_ARG_NONE, &debug, 0,
+	{ "debug",    '\0', 0, G_OPTION_ARG_NONE, &debug,
 	  "Print debugging output",          NULL},
-#ifndef USE_POPT_DLL
-	POPT_AUTOHELP
-#endif
+	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_STRING_ARRAY, &args,
+	  NULL,	NULL},
 	{ NULL }
 };
 
@@ -634,7 +632,7 @@ gpa_test_pairs (GPANode *config, const GpaTestPair *tests, gint max)
 
 
 static GpaTestRetval
-test_gpa_run_num (gint num)
+test_gpa_run_num (void)
 {
 	GpaTestRetval ret;
 
@@ -669,15 +667,19 @@ usage (gchar *error)
 }
 
 static void
-parse_command_line (int argc, const char ** argv, gint *num)
+parse_command_line (gint *argc, gchar *** argv)
 {
-	poptContext popt;
-	char **args;
+	GOptionContext *goption_context;
+	GError *error = NULL;
 
-	popt = poptGetContext ("test_gpa", argc, argv, options, 0);
-	poptGetNextOpt (popt);
+	goption_context = g_option_context_new ("NODE...");
+	g_option_context_add_main_entries (goption_context, options, NULL);
+	g_option_context_parse (goption_context, argc, argv, &error);
+	g_option_context_free (goption_context);
+
+	if (error != NULL)
+		usage(error->message);
 
-	args = (char**) poptGetArgs (popt);
 	if (args) {
 		gint i;
 		for (i = 0; args[i]; i++) {
@@ -689,14 +691,12 @@ parse_command_line (int argc, const char ** argv, gint *num)
 	if (dump_config || dump_root)
 		return;
 	
-	if (!num_str || num_str[0] == '\0') {
+	if (num == G_MININT) { /* Unspecified */
 		dump_config = TRUE;
 		return;
 	}
 
-	*num = atoi (num_str);
-
-	if (*num == -1) {
+	if (num == -1) {
 		GList *list = NULL;
 		/* We crash, this is part of the sanity check that allows us verify that crashes
 		 * are treated as errors when they happen (Chema)*/
@@ -704,7 +704,7 @@ parse_command_line (int argc, const char ** argv, gint *num)
 		list->next = NULL;
 	}
 	
-	if (*num < 0 || (*num > max_num)) {
+	if (num < 0 || (num > max_num)) {
 		gchar *error;
 		error = g_strdup_printf ("Num number out of range. Valid range is -1 to %d",
 					 max_num);
@@ -712,9 +712,7 @@ parse_command_line (int argc, const char ** argv, gint *num)
 	}
 	
 	if (debug)
-		g_print ("Num is %d\n", *num);
-	
-	poptFreeContext (popt);
+		g_print ("Num is %d\n", num);
 }
 
 static void
@@ -726,7 +724,7 @@ handle_sigsegv (int i)
 }
 
 int
-main (int argc, const char * argv[])
+main (int argc, char * argv[])
 {
 	GpaTestRetval ret;
 #ifndef G_OS_WIN32
@@ -736,7 +734,7 @@ main (int argc, const char * argv[])
 	g_type_init ();
 	loop = g_main_loop_new (NULL, FALSE);
 
-	parse_command_line (argc, argv, &num);
+	parse_command_line (&argc, &argv);
 
 #ifndef G_OS_WIN32
 	/* Catch sigsegv signals */
@@ -781,7 +779,7 @@ main (int argc, const char * argv[])
 		return TEST_GPA_SUCCESS;
 	}
 
-	ret = test_gpa_run_num (num);
+	ret = test_gpa_run_num ();
 		
 	return ret;
 }
